/* Redirect.  Simple mapping which alters dst to a local IP address. */
#include <linux/types.h>
#include <linux/ip.h>
#include <linux/timer.h>
#include <linux/module.h>
#include <linux/netfilter.h>
#include <linux/netdevice.h>
#include <linux/if.h>
#include <linux/inetdevice.h>
#include <net/protocol.h>
#include <net/checksum.h>
#include <linux/netfilter_ipv4.h>
#include "NAT/ip_nat_rule.h"

EXPORT_NO_SYMBOLS;

static int
redirect_check(enum ip_nat_manip_type maniptype,
	       const struct ip_nat_rule *rule)
{
	if (maniptype != IP_NAT_MANIP_DST) {
		DEBUGP("ipnat_rule_redirect_check: must be DST manip.\n");
		return 0;
	}

	return 1;
}

static unsigned int
redirect_setup(struct ip_conntrack *conntrack,
	       struct ip_nat_info *info,
	       unsigned int hooknum,
	       struct sk_buff *skb,
	       struct ip_nat_multi_range *mr,
	       void *moptions)
{
	unsigned int ret;
	u_int32_t newdst;
	struct ip_nat_multi_range *newrange;
	unsigned int i;

	/* Local packets: make them go to loopback */
	if (hooknum == NF_IP_LOCAL_OUT)
		newdst = htonl(0x7F000001);
	else
		/* Grab first address on interface. */
		newdst = (((struct in_device *)skb->dev->ip_ptr)
			  ->ifa_list->ifa_local);

	/* Copy entire range: consider multiple port specs. */
	newrange = kmalloc(sizeof(struct ip_nat_multi_range)
			   + sizeof(struct ip_nat_range) * (mr->rangesize-1),
			   GFP_ATOMIC);
	if (!newrange) {
		DEBUGP("Redirect: oom.\n");
		return NF_DROP;
	}

	/* Transfer from original range. */
	memcpy(newrange, mr,
	       sizeof(struct ip_nat_multi_range)
	       + sizeof(struct ip_nat_range) * (mr->rangesize-1));

	/* Substitute IPs for new IP dst. */
	for (i = 0; i < mr->rangesize; i++) {
		newrange->range[i].flags |= IP_NAT_RANGE_MAP_IPS;
		newrange->range[i].max_ip = newrange->range[i].min_ip
			= newdst;
	}

	/* Hand modified range to generic setup. */
	ret = ip_nat_setup_info(conntrack, info, newrange, hooknum);
	kfree(newrange);
	return ret;
}

static struct ip_nat_mapping_type mapping_redirect
= { { NULL, NULL }, "redirect", NETFILTER_VERSION, THIS_MODULE,
    redirect_check,
    redirect_setup,
    0
};

int init_module(void)
{
	printk("ip_nat_map_redirect v%s\n", NETFILTER_VERSION);
	return ip_nat_register_mapping_type(&mapping_redirect);
}

void cleanup_module(void)
{
	ip_nat_unregister_mapping_type(&mapping_redirect);
}
