/* Static.  Simple mapping which forces a 1:1, IP address only
 * mapping.  You can do this (faster) with Alexey's NAT, but then you
 * lose ftp magic etc.
 */
#include <linux/types.h>
#include <linux/ip.h>
#include <linux/timer.h>
#include <linux/module.h>
#include <linux/netfilter.h>
#include <net/protocol.h>
#include <net/checksum.h>
#include <linux/netfilter_ipv4.h>
#include "NAT/ip_nat_rule.h"

EXPORT_NO_SYMBOLS;

static int
static_check(enum ip_nat_manip_type maniptype, const struct ip_nat_rule *rule)
{
	/* Maybe one day we'll allow merging of mappings with moptions,
	   so we'll check here. */
	if (rule->mr.rangesize != 1)
		return 0;

	return 1;
}

static unsigned int
static_setup(struct ip_conntrack *conntrack,
	     struct ip_nat_info *info,
	     unsigned int hooknum,
	     struct sk_buff *skb,
	     struct ip_nat_multi_range *mr,
	     void *moptions)
{
	struct ip_nat_multi_range newrange;
	u_int32_t new_ip, curr_ip
		= (hooknum == NF_IP_PRE_ROUTING
		   ? conntrack->tuplehash[0].tuple.dst.ip
		   : conntrack->tuplehash[0].tuple.src.ip);

	/* Base is as given in the options. */
	new_ip = mr->range[0].min_ip + (curr_ip - *((u_int32_t *)moptions));

	if (new_ip > mr->range[0].max_ip)
		/* Don't color outside the lines. */
		return NF_DROP;

	/* Specify new range EXACTLY (ie. only IP to be mapped). */
	newrange.rangesize = 1;
	newrange.range[0].flags
		= IP_NAT_RANGE_MAP_IPS | IP_NAT_RANGE_PROTO_SPECIFIED;
	newrange.range[0].min_ip = newrange.range[0].max_ip
		= new_ip;
	/* LOCAL_OUT means source manip */
	if (hooknum == NF_IP_LOCAL_OUT) {
		newrange.range[0].min = newrange.range[0].max
			= conntrack->tuplehash[0].tuple.src.u;
	} else {
		/* In un-NAT'ed conntrack, reply tuple is inverse. */
		newrange.range[0].min = newrange.range[0].max
			= conntrack->tuplehash[1].tuple.src.u;
	}

	/* Hand modified range to generic setup. */
	return ip_nat_setup_info(conntrack, info, &newrange, hooknum);
}

static struct ip_nat_mapping_type mapping_static
= { { NULL, NULL }, "static", NETFILTER_VERSION, THIS_MODULE,
    static_check,
    static_setup,
    sizeof(u_int32_t)
};

int init_module(void)
{
	printk("ip_nat_map_static v%s\n", NETFILTER_VERSION);
	return ip_nat_register_mapping_type(&mapping_static);
}

void cleanup_module(void)
{
	ip_nat_unregister_mapping_type(&mapping_static);
}
