/* UDP protocol handling shared library. */
#include <getopt.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <netdb.h>
#include <netinet/in.h>
#include <net/if.h>
#include <getopt.h>

#include "NAT/ip_nat_rule.h"
#include "NAT/ipnatctl.h"

static struct option options[] =
{
	{ "source-port", 1, 0, '1' },
		{ "sport", 1, 0, '1' }, /* alias */
	{ "destination-port", 1, 0, '2' },
		{ "dport", 1, 0, '2' }, /* alias */
	{ "to-ports", 1, 0, '3' },
	{ NULL, 0, 0, 0 }
};

static void help(void)
{
	fprintf(stderr,
		"UDP <input spec> options:\n"
		"  --source-port <port>\n"
		"  --sport <port>		Source port\n"
		"  --destination-port <port>\n"
		"  --dport <port>		Destination port\n\n");

	fprintf(stderr,
		"UDP <output range> options:\n"
		"  --to-ports <port>[-<port>]\n\n");
}

/* Fills in ret in network order. */
static int get_port(const char *port, const char *proto, u_int16_t *ret)
{
	struct servent *service;
	int pnum;

	service = getservbyname(port, proto);
	if (service)
		*ret = service->s_port;
	else {
		pnum = atoi(port);
		if (pnum < 1 || pnum > 65535)
			return 0;
		*ret = htons(pnum);
	}
	return 1;
}

static int
get_port_range(const char *ports, const char *proto,
	       u_int16_t *min, u_int16_t *max)
{
	char *delim = strchr(ports, '-');

	if (!delim) {
		int ret = get_port(ports, proto, min);
		*max = *min;
		return ret;
	}

	*delim = '\0';

	return (get_port(ports, proto, min) && get_port(delim+1, proto, max));
}

static int parse(int c, char **argv, struct ip_nat_rule_user *rule)
{
	switch (c) {
	case '1':
		if (!get_port(optarg, "udp",&rule->match.tuple.src.u.udp.port))
			print_usage("Bad UDP src port number `%s'", optarg);

		rule->mask.tuple.src.u.udp.port = 0xFFFF;
		break;

	case '2':
		if (!get_port(optarg, "udp",&rule->match.tuple.dst.u.udp.port))
			print_usage("Bad UDP dest port number `%s'", optarg);

		rule->mask.tuple.dst.u.udp.port = 0xFFFF;
		break;

	case '3':	
		rule->range.flags |= IP_NAT_RANGE_PROTO_SPECIFIED;
		if (!get_port_range(optarg, "udp",
				    &rule->range.min.udp.port,
				    &rule->range.max.udp.port))
			print_usage("Bad UDP port range `%s'", optarg);
		break;

	default:
		return 0;
	}

	return 1;
}

static struct ipnat_proto udp_proto
= { NULL,
    IPPROTO_UDP,
    help,
    parse,
    options
};

void _init(void)
{
	register_protocol(&udp_proto);
}
