/* Simple kernel module to refragment packets forwards */
#include <linux/module.h>
#include <linux/netfilter.h>
#include <linux/netfilter_ipv4.h>
#include <linux/ip.h>
#include <net/ip.h>

struct net_device;

/* The work comes in here from netfilter.c. */
static unsigned int
forward_frag_hook(unsigned int hook,
		  struct sk_buff **pskb,
		  const struct net_device *indev,
		  const struct net_device *outdev)
{
	(*pskb)->nfcache |= NFC_IP_FRAG;

	if ((*pskb)->nh.iph->frag_off & __constant_htons(IP_MF|IP_OFFSET)) {
		*pskb = ip_defrag(*pskb);
		if (*pskb) {	
			ip_send_check((*pskb)->nh.iph);
			(*pskb)->nfcache |= NFC_UNKNOWN | NFC_ALTERED;

			ip_fragment(*pskb, (*pskb)->dst->output);
		}
		return NF_STOLEN;
	}
	else return NF_ACCEPT;
}

static struct nf_hook_ops forward_frag_ops
= { { NULL, NULL }, forward_frag_hook, NULL, PF_INET, NF_IP_LOCAL_OUT, -100 };

int init_module(void)
{
	return nf_register_hook(&forward_frag_ops);
}

void cleanup_module(void)
{
	nf_unregister_hook(&forward_frag_ops);
}
