#ifndef _IP_NAT_RULE_H
#define _IP_NAT_RULE_H
#include "ip_nat.h"

struct ip_nat_rule;
struct net_device;

#define IP_NAT_BASE_CTL  	81	/* base for NAT socket options */
#define IP_NAT_SO_SET_INSERT	IP_NAT_BASE_CTL
#define IP_NAT_SO_SET_DELETE	(IP_NAT_BASE_CTL+1)
#define IP_NAT_SO_SET_MAX	IP_NAT_SO_SET_DELETE

#define IP_NAT_SO_GET_VERSION	IP_NAT_BASE_CTL
#define IP_NAT_SO_GET_MAX	IP_NAT_SO_GET_VERSION

#define IP_NAT_MAPPING_TYPE_MAX_NAMELEN 16

struct ip_nat_rule_match
{
	struct ip_conntrack_tuple tuple;
	/* Interface, and interface mask
	   (src manip = outgoing iface, dst manip = incoming iface). */
	char ifname[IFNAMSIZ];
};

struct ip_nat_rule_user
{
	struct ip_nat_rule_match match;
	struct ip_nat_rule_match mask;

	/* Type of manipulation. */
	enum ip_nat_manip_type maniptype;

	/* Range to map into */
	struct ip_nat_range range;

	/* Nul-terminated string. */
	char typename[IP_NAT_MAPPING_TYPE_MAX_NAMELEN];

	/* Mapping options (if any). */
	unsigned char map_options[0];
};

#ifdef __KERNEL__
struct ip_nat_mapping_type
{
	struct list_head list;

	/* Name of this mapping. */
	const char name[IP_NAT_MAPPING_TYPE_MAX_NAMELEN];

	const char *version;

	/* If you want your module count modified as rules use the type,
	   set this. */
	struct module *me;

	/* Will it accept this rule?  Return true or false. */
	int (*check)(enum ip_nat_manip_type maniptype,
		     const struct ip_nat_rule *rule);

	/* Function to call if we get a match. */
	unsigned int (*setup)(struct ip_conntrack *conntrack,
			      struct ip_nat_info *info,
			      unsigned int hooknum,
			      struct sk_buff *skb,
			      struct ip_nat_multi_range *mr,
			      void *moptions);

	/* Size of options. */
	size_t optionlen;
};

struct ip_nat_rule
{
	struct list_head list;

	/* Use count: current references to this rule + 1 home
           reference. */
	atomic_t use;

	/* We're dead; ignore us. */
	int dead;

	/* Here's what we match. */
	struct ip_nat_rule_match match;
	struct ip_nat_rule_match mask;

	/* Our pre-calculated precedence number. */
	u_int32_t precedence;

	/* Pointer to mapping type. */
	struct ip_nat_mapping_type *mtype;
	void *mopts;

	/* Range to map onto */
	struct ip_nat_multi_range mr;
};

extern int ip_nat_register_mapping_type(struct ip_nat_mapping_type *me);
extern void ip_nat_unregister_mapping_type(struct ip_nat_mapping_type *me);

/* Get reference to rule. */
extern struct ip_nat_rule *
ip_nat_rule_get(const struct ip_conntrack_tuple *tuple,
		const char *ifname,
		unsigned int hooknum);
/* Already-locked version */
extern struct ip_nat_rule *
__ip_nat_rule_get(const struct ip_conntrack_tuple *tuple,
		  const char *ifname,
		  unsigned int hooknum);

/* Drop reference to rule. */
extern void ip_nat_rule_put(struct ip_nat_rule *rule);

extern int ip_nat_rule_init(void) __init;
extern void ip_nat_rule_cleanup(void);
#endif /* __KERNEL__ */
#endif /* _IP_NAT_RULE_H */
