#ifndef __LINUX_NETFILTER_DEV_H
#define __LINUX_NETFILTER_DEV_H
#include <linux/netfilter.h>

struct nfdev_verdict
{
	/* NF_ACCEPT, NF_DROP, NF_QUEUE */
	unsigned int verdict;

	/* New packet length. */
	u_int32_t pktlen;

	/* Mark value. */
	unsigned long nfmark;

	/* Reason: explains mark value. */
	u_int32_t nfreason;
};

struct nfdev_condition
{
	/* Bitmask of hook numbers to match (1 << hooknum). */
	unsigned int hookmask;
	/* If non-zero, only catch packets with this mark. */
	unsigned int mark;
	/* If non-zero, only catch packets of this reason. */
	unsigned int reason;
};

/* When opened, the device has a (deletable) condition of {0xFFFFFFFF, 0, 0},
 * ie. catch anything, but it's appended to the end of the list. */
#define NFD_IOC_MAGIC 0xAD
#define NFDIOSVERDICT _IOW(NFD_IOC_MAGIC,0,sizeof(struct nfdev_verdict))
#define NFDIOSADDCOND _IOW(NFD_IOC_MAGIC,1,sizeof(struct nfdev_condition))
#define NFDIOSDELCOND _IOW(NFD_IOC_MAGIC,2,sizeof(struct nfdev_condition))

struct nfdev_head
{
	char iniface[IFNAMSIZ];
	char outiface[IFNAMSIZ];
	/* Mark value. */
	unsigned long mark;
	/* Total packet length. */
	u_int32_t pktlen;
	/* Reason. */
	u_int32_t reason;
	/* And the hook you came in on. */
	u_int32_t hook;	
};

/* This is what you get from a read. */
struct nfdev_info
{
	/* These are not writable. */
	struct nfdev_head head;

	/* Here down is writable (actual packet length). */
	char packet[0];
};
#endif /*__LINUX_NETFILTER_DEV_H*/



