/* Kernel module to match connection tracking information.
 * GPL (C) 1999  Rusty Russell (rusty@rustcorp.com.au).
 */
#include <linux/module.h>
#include <linux/skbuff.h>
#include "conntrack/ip_conntrack.h"
#include "packet-filter/kernel/ip_tables.h"
#include "packet-filter/extensions/ipt_state.h"
EXPORT_NO_SYMBOLS;

static int
match(struct sk_buff *skb,
      const char *indev,
      const char *outdev,
      const union ipt_matchinfo *matchinfo,
      int offset,
      const void *hdr,
      u_int16_t datalen,
      int *hotdrop)
{
	const struct ipt_state_info *sinfo = (void *)matchinfo->other;
	enum ip_conntrack_info ctinfo;
	struct ip_conntrack_tuple_hash *h;

	DEBUGP("ipt_state: getting state for skb %p\n", skb);
	ctinfo = ip_conntrack_get(skb, &h);
	DEBUGP("ipt_state: got state %u for skb %p\n", ctinfo, skb);
	if (ctinfo != IP_CT_INVALID)
		ip_conntrack_put(h->ctrack);

	return (sinfo->statemask & IPT_STATE_BIT(ctinfo));
}

static struct ipt_match state_match
= { { NULL, NULL }, "state", NETFILTER_VERSION,
    &match, NULL, NULL, THIS_MODULE };

int __init init(void)
{
	__MOD_INC_USE_COUNT(ip_conntrack_module);
	return ipt_register_match(&state_match);
}

void __exit cleanup(void)
{
	ipt_unregister_match(&state_match);
	__MOD_DEC_USE_COUNT(ip_conntrack_module);
}

module_init(init);
module_exit(cleanup);
