/* Shared library add-on to iptables to add QUEUE support. */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <getopt.h>
#include "packet-filter/userspace/iptables.h"
#include "packet-filter/kernel/ip_tables.h"
#include "packet-filter/extensions/ipt_QUEUE.h"

/* Function which prints out usage message. */
static void
help(void)
{
	printf(
"QUEUE v%s options:\n"
" --mark MARK		Number to mark skb with\n",
NETFILTER_VERSION);
}

static struct option opts[] = {
	{ "mark", 1, 0, '!' },
	{ 0 }
};

static void
init(union ipt_targinfo *targinfo, unsigned int *nfcache)
{
	struct ipt_queue_info *info = (struct ipt_queue_info *)targinfo->other;

	info->mark = 0;
}

/* Function which parses command options; returns true if it
   ate an option */
static int
parse(int c, char **argv, int invert, unsigned int *flags,
      struct ipt_entry *entry)
{
	struct ipt_queue_info *info =
		(struct ipt_queue_info *)entry->targinfo.other;

	switch (c) {
	case '!': {
		char *end;

		info->mark = strtol(optarg, &end, 0);
		if (*end != '\0' || end == optarg)
			exit_error(PARAMETER_PROBLEM,
				   "Invalid QUEUE --mark: %s\n", optarg);
	}
	break;

	default:
		return 0;
	}

	return 1;
}

/* Final check; nothing. */
static void final_check(unsigned int flags)
{
}

/* Prints out the union ipt_targinfo. */
static void print(const struct ipt_entry *entry, int numeric)
{
	struct ipt_queue_info *queueinfo
		= (struct ipt_queue_info *)entry->targinfo.other;

	printf("QUEUE ");
	if (queueinfo->mark)
		printf("mark %li ", queueinfo->mark);
}

struct iptables_target queue
= { NULL,
    "QUEUE",
    NETFILTER_VERSION,
    &help,
    &init,
    &parse,
    &final_check,
    &print,
    opts
};

void _init(void)
{
	register_target(&queue);
}
