#! /bin/sh
# Fragment detection and avoidance test.  Also covers truncated packets.

# Need job control.
set -m

## First: simple `match frag rule' test.
../packet-filter/userspace/iptables -P FORWARD ACCEPT
../packet-filter/userspace/iptables -A FORWARD -f -j ACCEPT || exit 1

../tools/rcv_ip 2 2 < /dev/tap1 > $TMPFILE &
sleep 1

../tools/gen_ip FRAG=8 $TAP0NET.2 $TAP1NET.2 100 1 8 0 55 57 > /dev/tap0
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 1 8 0 56 58 > /dev/tap0

if wait %../tools/rcv_ip; then :
else
    echo ../tools/rcv_ip failed:
    cat $TMPFILE
    exit 1
fi

for LINE in "FRAG=8 $TAP0NET.2 $TAP1NET.2 100 1" \
    "$TAP0NET.2 $TAP1NET.2 100 1 8 0 56 58"
do
    read ACTUAL
    if [ "$LINE" != "$ACTUAL" ]
    then
	echo Unexpected rcv: \`"$ACTUAL"\'.
	exit 1
    fi
done < $TMPFILE

if ../packet-filter/userspace/iptables -L FORWARD -v | head -3 | tail +1 | grep '^    1   100 ' >/dev/null
then
    :
else
    echo Bad Packet count:
    ../packet-filter/userspace/iptables -L FORWARD -v
    exit 1
fi
../packet-filter/userspace/iptables -D FORWARD -f -j ACCEPT || exit 1

# ICMP fragment test.
insmod ../packet-filter/extensions/ipt_icmp.o || exit 1
../packet-filter/userspace/iptables -A FORWARD -p ICMP ! --icmp-type 7 -j ACCEPT || exit 1

../tools/rcv_ip 2 2 < /dev/tap1 > $TMPFILE &
sleep 1

../tools/gen_ip FRAG=8 $TAP0NET.2 $TAP1NET.2 100 1 8 0 55 57 > /dev/tap0
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 1 8 0 56 58 > /dev/tap0

if wait %../tools/rcv_ip; then :
else
    echo ../tools/rcv_ip failed:
    cat $TMPFILE
    exit 1
fi

if ../packet-filter/userspace/iptables -L FORWARD -v | head -3 | tail +1 | grep '^    1   100 ' >/dev/null
then
    :
else
    echo Bad Packet count:
    ../packet-filter/userspace/iptables -L FORWARD -v
    exit 1
fi

../tools/rcv_ip 2 1 < /dev/tap1 > $TMPFILE &
sleep 1

# ICMP truncated test; should hotdrop packet.
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 21 1 7 0 > /dev/tap0

if wait %../tools/rcv_ip
then
    echo ../tools/rcv_ip got packet:
    cat $TMPFILE
    exit 1
fi

if ../packet-filter/userspace/iptables -L FORWARD -v | head -3 | tail +1 | grep '^    1   100 ' >/dev/null
then
    :
else
    echo Bad Packet count:
    ../packet-filter/userspace/iptables -L FORWARD -v
    exit 1
fi

../packet-filter/userspace/iptables -D FORWARD -p ICMP ! --icmp-type 7 -j ACCEPT || exit 1
rmmod ipt_icmp || exit 1


# UDP fragment test.
insmod ../packet-filter/extensions/ipt_udp.o || exit 1
../packet-filter/userspace/iptables -A FORWARD -p UDP ! --sport 15 -j ACCEPT || exit 1

../tools/rcv_ip 2 2 < /dev/tap1 > $TMPFILE &
sleep 1

../tools/gen_ip FRAG=8 $TAP0NET.2 $TAP1NET.2 100 17 5 7 > /dev/tap0
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 17 6 8 > /dev/tap0

if wait %../tools/rcv_ip; then :
else
    echo ../tools/rcv_ip failed:
    cat $TMPFILE
    exit 1
fi

if ../packet-filter/userspace/iptables -L FORWARD -v | head -3 | tail +1 | grep '^    1   100 ' >/dev/null
then
    :
else
    echo Bad UDP packet count:
    ../packet-filter/userspace/iptables -L FORWARD -v
    exit 1
fi

../tools/rcv_ip 2 1 < /dev/tap1 > $TMPFILE &
sleep 1

# UDP truncated test; should hotdrop packet.
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 27 17 5 7 > /dev/tap0

if wait %../tools/rcv_ip
then
    echo ../tools/rcv_ip got UDP packet:
    cat $TMPFILE
    exit 1
fi

if ../packet-filter/userspace/iptables -L FORWARD -v | head -3 | tail +1 | grep '^    1   100 ' >/dev/null
then
    :
else
    echo Bad UDP packet count:
    ../packet-filter/userspace/iptables -L FORWARD -v
    exit 1
fi

../packet-filter/userspace/iptables -D FORWARD -p UDP ! --sport 15 -j ACCEPT || exit 1
rmmod ipt_udp || exit 1


# TCP fragment test.
insmod ../packet-filter/extensions/ipt_tcp.o || exit 1
../packet-filter/userspace/iptables -A FORWARD -p TCP ! --sport 15 -j ACCEPT || exit 1

../tools/rcv_ip 2 2 < /dev/tap1 > $TMPFILE &
sleep 1

../tools/gen_ip FRAG=16 $TAP0NET.2 $TAP1NET.2 100 6 5 7 SYN > /dev/tap0
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 6 6 8 SYN > /dev/tap0

if wait %../tools/rcv_ip; then :
else
    echo ../tools/rcv_ip failed:
    cat $TMPFILE
    exit 1
fi

if ../packet-filter/userspace/iptables -L FORWARD -v | head -3 | tail +1 | grep '^    1   100 ' >/dev/null
then
    :
else
    echo Bad TCP packet count:
    ../packet-filter/userspace/iptables -L FORWARD -v
    exit 1
fi

../tools/rcv_ip 2 1 < /dev/tap1 > $TMPFILE &
sleep 1

# TCP truncated test; should hotdrop packets.
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 27 6 5 7 SYN > /dev/tap0
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 35 6 5 7 SYN > /dev/tap0
../tools/gen_ip FRAG=8 $TAP0NET.2 $TAP1NET.2 100 6 5 7 SYN > /dev/tap0

if wait %../tools/rcv_ip
then
    echo ../tools/rcv_ip got TCP packet:
    cat $TMPFILE
    exit 1
fi

if ../packet-filter/userspace/iptables -L FORWARD -v | head -3 | tail +1 | grep '^    1   100 ' >/dev/null
then
    :
else
    echo Bad TCP packet count:
    ../packet-filter/userspace/iptables -L FORWARD -v
    exit 1
fi

../packet-filter/userspace/iptables -D FORWARD -p TCP ! --sport 15 -j ACCEPT || exit 1
rmmod ipt_tcp || exit 1

exit 0
