#! /bin/sh

# Note that this tests the INSTALLED shared libs, not the ones in the
# tree. 8-(

# Need job control.
set -m

# Test the various options for the limit match.
insmod ../packet-filter/extensions/ipt_limit.o || exit 1

../packet-filter/userspace/iptables -A FORWARD -m limit --limit 100/sec || exit 1

if [ "`../packet-filter/userspace/iptables -L FORWARD | grep '^ '`" != "           all  --  anywhere             anywhere           limit: avg 100/sec burst 5 " ]
then
    echo Unexpected ouptut from \`-m limit --limit 100/sec\'
    ../packet-filter/userspace/iptables -L FORWARD
    exit 1
fi

../packet-filter/userspace/iptables -F FORWARD || exit 1
../packet-filter/userspace/iptables -A FORWARD -m limit --limit 100 || exit 1

if [ "`../packet-filter/userspace/iptables -L FORWARD | grep '^ '`" != "           all  --  anywhere             anywhere           limit: avg 100/sec burst 5 " ]
then
    echo Unexpected ouptut from \`-m limit --limit 100\'
    ../packet-filter/userspace/iptables -L FORWARD
    exit 1
fi

../packet-filter/userspace/iptables -F FORWARD || exit 1
../packet-filter/userspace/iptables -A FORWARD -m limit --limit 100/min --limit-burst 10 || exit 1

if [ "`../packet-filter/userspace/iptables -L FORWARD | grep '^ '`" != "           all  --  anywhere             anywhere           limit: avg 100/min burst 10 " ]
then
    echo Unexpected ouptut from \`-m limit --limit 100/min --limit-burst 10/m\'
    ../packet-filter/userspace/iptables -L FORWARD
    exit 1
fi

../packet-filter/userspace/iptables -F FORWARD || exit 1
../packet-filter/userspace/iptables -A FORWARD -m limit --limit 100/hour --limit-burst 1 || exit 1

if [ "`../packet-filter/userspace/iptables -L FORWARD | grep '^ '`" != "           all  --  anywhere             anywhere           limit: avg 100/hour burst 1 " ]
then
    echo Unexpected ouptut from \`--limit 100/hour --limit-burst 1\'
    ../packet-filter/userspace/iptables -L FORWARD
    exit 1
fi

../packet-filter/userspace/iptables -F FORWARD || exit 1
../packet-filter/userspace/iptables -A FORWARD -m limit --limit 1/day --limit-burst 2 || exit 1

if [ "`../packet-filter/userspace/iptables -L FORWARD | grep '^ '`" != "           all  --  anywhere             anywhere           limit: avg 1/day burst 2 " ]
then
    echo Unexpected ouptut from \`--limit 1/day --limit-burst 2\'
    ../packet-filter/userspace/iptables -L FORWARD
    exit 1
fi

../packet-filter/userspace/iptables -F FORWARD || exit 1
../packet-filter/userspace/iptables -P FORWARD DROP || exit 1
../packet-filter/userspace/iptables -A FORWARD -m limit --limit 1/day --limit-burst 2 -j ACCEPT || exit 1

# Third one should be dropped.
../tools/rcv_ip 2 2 < /dev/tap1 > $TMPFILE &
sleep 1

../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 1 8 0 56 58 > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.3 100 1 8 0 56 58 > /dev/tap0 || exit 1

if wait %../tools/rcv_ip; then :
else 
    echo ../tools/rcv_ip failed:
    cat $TMPFILE
    exit 1
fi

if [ "`cat $TMPFILE`" != "$TAP0NET.2 $TAP1NET.2 100 1 8 0 56 58 
$TAP0NET.2 $TAP1NET.3 100 1 8 0 56 58 " ]
then
    echo Unexpected packets received:
    echo \`"`cat $TMPFILE`"\'
    exit 1
fi

# Third one should be dropped.
../tools/rcv_ip 2 1 < /dev/tap1 > $TMPFILE &
sleep 1

../tools/gen_ip $TAP0NET.2 $TAP1NET.4 100 1 8 0 56 58 > /dev/tap0 || exit 1

if wait %../tools/rcv_ip
then
    echo ../tools/rcv_ip not limited:
    cat $TMPFILE
    exit 1
fi

../packet-filter/userspace/iptables -F FORWARD || exit 1
rmmod ipt_limit || exit 1

exit 0
