#! /bin/sh 

# Need job control.
set -m

# Test the various options for the state match.
insmod ../conntrack/ip_conntrack.o || exit 1
insmod ../packet-filter/extensions/ipt_state.o || exit 1

# Must force state.
../packet-filter/userspace/iptables -A FORWARD -m state 2>/dev/null && exit 1

# Accept NEW state
../packet-filter/userspace/iptables -P FORWARD DROP || exit 1
../packet-filter/userspace/iptables -A FORWARD -m state --state new -j ACCEPT || exit 1

# Send NEW packet through
../tools/rcv_ip 2 1 < /dev/tap1 > $TMPFILE &
sleep 1

../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 1 8 0 55 57 > /dev/tap0 || exit 1
if wait %../tools/rcv_ip; then :
else
    echo rcv_ip failed:
    cat $TMPFILE
    exit 1
fi

# Send reply packet through
../tools/rcv_ip 2 1 < /dev/tap0 > $TMPFILE &
sleep 1

../tools/gen_ip $TAP1NET.2 $TAP0NET.2 100 1 0 0 55 57 > /dev/tap1 || exit 1
if wait %../tools/rcv_ip
then 
    echo rcv_ip succeeded:
    cat $TMPFILE
    exit 1
fi

# Accept ESTABLISHED state
../packet-filter/userspace/iptables -R FORWARD 1 -m state --state established -j ACCEPT || exit 1

# Send NEW packet through
../tools/rcv_ip 2 1 < /dev/tap1 > $TMPFILE &
sleep 1

../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 1 8 0 55 57 > /dev/tap0 || exit 1
if wait %../tools/rcv_ip
then 
    echo rcv_ip for established succeeded:
    cat $TMPFILE
    exit 1
fi

# Send reply packet through
../tools/rcv_ip 2 1 < /dev/tap0 > $TMPFILE &
sleep 1

../tools/gen_ip $TAP1NET.2 $TAP0NET.2 100 1 0 0 55 57 > /dev/tap1 || exit 1
if wait %../tools/rcv_ip; then :
else
    echo rcv_ip for established failed:
    cat $TMPFILE
    exit 1
fi

# Accept RELATED state
../packet-filter/userspace/iptables -R FORWARD 1 -m state --state related -j ACCEPT || exit 1

# Send NEW packet through
../tools/rcv_ip 2 1 < /dev/tap1 > $TMPFILE &
sleep 1

../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 1 8 0 55 57 > /dev/tap0 || exit 1
if wait %../tools/rcv_ip
then 
    echo rcv_ip for related succeeded:
    cat $TMPFILE
    exit 1
fi


# Send ERROR reply packet through
../tools/rcv_ip 2 1 < /dev/tap0 > $TMPFILE &
sleep 1

../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 1 8 0 55 57 | ../tools/gen_err $TAP1NET.2 11 0 > /dev/tap1 || exit 1

if wait %../tools/rcv_ip; then :
else
    echo rcv_ip for related failed:
    cat $TMPFILE
    exit 1
fi

# Accept INVALID state
../packet-filter/userspace/iptables -R FORWARD 1 -m state --state invalid -j ACCEPT || exit 1

# Send pong packet through
../tools/rcv_ip 2 1 < /dev/tap1 > $TMPFILE &
sleep 1

../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 1 0 0 56 58 > /dev/tap0 || exit 1
if wait %../tools/rcv_ip; then :
else 
    echo rcv_ip for invalid failed:
    cat $TMPFILE
    exit 1
fi

# Multiple states
../packet-filter/userspace/iptables -R FORWARD 1 -m state --state new,related -j ACCEPT || exit 1

# Send NEW packet through
../tools/rcv_ip 2 1 < /dev/tap1 > $TMPFILE &
sleep 1

../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 1 8 0 65 67 > /dev/tap0 || exit 1
if wait %../tools/rcv_ip
then :
else
    echo rcv_ip for new,related NEW failed:
    cat $TMPFILE
    exit 1
fi

# Send ERROR reply packet through
../tools/rcv_ip 2 1 < /dev/tap0 > $TMPFILE &
sleep 1

../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 1 8 0 65 67 | ../tools/gen_err $TAP1NET.2 11 0 > /dev/tap1 || exit 1

if wait %../tools/rcv_ip; then :
else
    echo rcv_ip for new,related failed:
    cat $TMPFILE
    exit 1
fi

# Send INVALID reply packet through
../tools/rcv_ip 2 1 < /dev/tap0 > $TMPFILE &
sleep 1

../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 1 8 0 66 68 | ../tools/gen_err $TAP1NET.2 11 0 > /dev/tap1 || exit 1

if wait %../tools/rcv_ip
then
    echo rcv_ip for new,related succeeded:
    cat $TMPFILE
    exit 1
fi

# Try inverse match.
# Accept ! ESTABLISHED state
../packet-filter/userspace/iptables -R FORWARD 1 -m state ! --state established -j ACCEPT || exit 1
../packet-filter/userspace/iptables -R FORWARD 1 -m state --state ! established -j ACCEPT || exit 1

# Send NEW packet through
../tools/rcv_ip 2 1 < /dev/tap1 > $TMPFILE &
sleep 1

../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 1 8 0 75 77 > /dev/tap0 || exit 1
if wait %../tools/rcv_ip
then :
else 
    echo rcv_ip for ! established failed:
    cat $TMPFILE
    exit 1
fi
	
../packet-filter/userspace/iptables -F FORWARD || exit 1
rmmod ipt_state || exit 1
rmmod ip_conntrack || exit 1
exit 0
