#! /bin/sh

# Need job control.
set -m

# Test the various options for the multiport match.
insmod ../packet-filter/extensions/ipt_multiport.o || exit 1

# Too many ports doesn't work.
../packet-filter/userspace/iptables -A FORWARD -p tcp -m multiport --sports 1,2,3,4,5,6,7,8 2>/dev/null && exit 1
../packet-filter/userspace/iptables -A FORWARD -p tcp -m multiport --dports 1,2,3,4,5,6,7,8 2>/dev/null && exit 1
../packet-filter/userspace/iptables -A FORWARD -p tcp -m multiport --ports 1,2,3,4,5,6,7,8 2>/dev/null && exit 1

# Mixing ports doesn't work.
../packet-filter/userspace/iptables -A FORWARD -p tcp -m multiport --sports 1,2 --dports 1,2 2>/dev/null && exit 1
../packet-filter/userspace/iptables -A FORWARD -p tcp -m multiport --ports 1,2 --dports 1,2 2>/dev/null && exit 1
../packet-filter/userspace/iptables -A FORWARD -p tcp -m multiport --sports 1,2 --ports 1,2 2>/dev/null && exit 1
../packet-filter/userspace/iptables -A FORWARD -p tcp -m multiport --sports 1,2 --dports 1,2 --ports 1,2 2>/dev/null && exit 1
../packet-filter/userspace/iptables -A FORWARD -p tcp -m multiport --sports 1,2 --sports 5,6 2>/dev/null && exit 1
../packet-filter/userspace/iptables -A FORWARD -p tcp -m multiport --dports 1,2 --dports 5,6 2>/dev/null && exit 1
../packet-filter/userspace/iptables -A FORWARD -p tcp -m multiport --ports 1,2 --ports 5,6 2>/dev/null && exit 1

# Need protocol first
../packet-filter/userspace/iptables -A FORWARD -m multiport --sports 1,4 2>/dev/null && exit 1
../packet-filter/userspace/iptables -A FORWARD -m multiport --dports 1,4 2>/dev/null && exit 1
../packet-filter/userspace/iptables -A FORWARD -m multiport --ports 1,4 2>/dev/null && exit 1
../packet-filter/userspace/iptables -A FORWARD -p icmp -m multiport --sports 1,4 2>/dev/null && exit 1
../packet-filter/userspace/iptables -A FORWARD -m multiport --sports 1,4 -p tcp 2>/dev/null && exit 1
../packet-filter/userspace/iptables -A FORWARD -m multiport --ports 1,4 -p tcp 2>/dev/null && exit 1

# Should work with tcp and udp before --sport or --dport.
../packet-filter/userspace/iptables -A FORWARD -p tcp -m multiport --ports 1,4 || exit 1
../packet-filter/userspace/iptables -A FORWARD -p udp -m multiport --ports 1,4 || exit 1
../packet-filter/userspace/iptables -A FORWARD -p tcp -m multiport --sports 1,4|| exit 1
../packet-filter/userspace/iptables -A FORWARD -p udp -m multiport --sports 1,4|| exit 1
../packet-filter/userspace/iptables -A FORWARD -p tcp -m multiport --dports 1,4|| exit 1
../packet-filter/userspace/iptables -A FORWARD -p udp -m multiport --dports 1,4|| exit 1

../packet-filter/userspace/iptables -A FORWARD -m multiport -p tcp --ports 1,4 || exit 1
../packet-filter/userspace/iptables -A FORWARD -m multiport -p udp --ports 1,4 || exit 1
../packet-filter/userspace/iptables -A FORWARD -m multiport -p tcp --sports 1,4 || exit 1
../packet-filter/userspace/iptables -A FORWARD -m multiport -p udp --sports 1,4 || exit 1
../packet-filter/userspace/iptables -A FORWARD -m multiport -p tcp --dports 1,4 || exit 1
../packet-filter/userspace/iptables -A FORWARD -m multiport -p udp --dports 1,4 || exit 1

../packet-filter/userspace/iptables -F FORWARD || exit 1

# Now we test --ports
../packet-filter/userspace/iptables -A FORWARD -m multiport -p udp --ports 1 -j ACCEPT || exit 1

../tools/rcv_ip 5 2 < /dev/tap1 > $TMPFILE &
sleep 1

../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 6 2 2 SYN > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 6 3 1 SYN > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 6 1 3 SYN > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 17 2 2 > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 17 3 1 > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 17 1 3 > /dev/tap0 || exit 1
if wait %../tools/rcv_ip; then :
else
    echo rcv_ip failed for udp --ports 1:
    cat $TMPFILE
    exit 1
fi
if [ "`cat $TMPFILE`" != "$TAP0NET.2 $TAP1NET.2 100 17 3 1 
$TAP0NET.2 $TAP1NET.2 100 17 1 3 " ]
then
    echo rcv_ip wrong for udp --ports 1:
    cat $TMPFILE
    exit 1
fi

../packet-filter/userspace/iptables -R FORWARD 1 -m multiport -p tcp --ports 1 -j ACCEPT || exit 1

../tools/rcv_ip 5 2 < /dev/tap1 > $TMPFILE &
sleep 1

../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 6 2 2 SYN > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 6 3 1 SYN > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 6 1 3 SYN > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 6 2 2 SYN > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 6 3 1 SYN > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 6 1 3 SYN > /dev/tap0 || exit 1

if wait %../tools/rcv_ip; then :
else
    echo rcv_ip failed for tcp --ports 1:
    cat $TMPFILE
    exit 1
fi
if [ "`cat $TMPFILE`" != "$TAP0NET.2 $TAP1NET.2 100 6 3 1 SYN 
$TAP0NET.2 $TAP1NET.2 100 6 1 3 SYN " ]
then
    echo rcv_ip wrong for tcp --ports 1:
    cat $TMPFILE
    exit 1
fi

# Now we test --sports
../packet-filter/userspace/iptables -R FORWARD 1 -m multiport -p udp --sports 1 -j ACCEPT || exit 1

../tools/rcv_ip 5 1 < /dev/tap1 > $TMPFILE &
sleep 1

../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 6 2 2 SYN > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 6 3 1 SYN > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 6 1 3 SYN > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 17 2 2 > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 17 3 1 > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 17 1 3 > /dev/tap0 || exit 1
if wait %../tools/rcv_ip; then :
else
    echo rcv_ip failed for udp --sports 1:
    cat $TMPFILE
    exit 1
fi
if [ "`cat $TMPFILE`" != "$TAP0NET.2 $TAP1NET.2 100 17 1 3 " ]
then
    echo rcv_ip wrong for udp --sports 1:
    cat $TMPFILE
    exit 1
fi

../packet-filter/userspace/iptables -R FORWARD 1 -m multiport -p tcp --sports 1 -j ACCEPT || exit 1

../tools/rcv_ip 5 1 < /dev/tap1 > $TMPFILE &
sleep 1

../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 17 2 2 > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 17 1 3 > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 17 3 1 > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 6 2 2 SYN > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 6 3 1 SYN > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 6 1 3 SYN > /dev/tap0 || exit 1

if wait %../tools/rcv_ip; then :
else
    echo rcv_ip failed for tcp --sports 1:
    cat $TMPFILE
    exit 1
fi
if [ "`cat $TMPFILE`" != "$TAP0NET.2 $TAP1NET.2 100 6 1 3 SYN " ]
then
    echo rcv_ip wrong for tcp --sports 1:
    cat $TMPFILE
    exit 1
fi

# Now we test --dports
../packet-filter/userspace/iptables -R FORWARD 1 -m multiport -p udp --dports 1 -j ACCEPT || exit 1

../tools/rcv_ip 5 1 < /dev/tap1 > $TMPFILE &
sleep 1

../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 17 2 2 > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 17 1 3 > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 17 3 1 > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 17 2 2 > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 17 1 3 > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 17 3 1 > /dev/tap0 || exit 1
if wait %../tools/rcv_ip; then :
else
    echo rcv_ip failed for udp --dports 1:
    cat $TMPFILE
    exit 1
fi
if [ "`cat $TMPFILE`" != "$TAP0NET.2 $TAP1NET.2 100 17 3 1 " ]
then
    echo rcv_ip wrong for udp --dports 1:
    cat $TMPFILE
    exit 1
fi

../packet-filter/userspace/iptables -R FORWARD 1 -m multiport -p tcp --dports 1 -j ACCEPT || exit 1

../tools/rcv_ip 5 1 < /dev/tap1 > $TMPFILE &
sleep 1

../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 17 2 2 > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 17 1 3 > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 17 3 1 > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 6 2 2 SYN > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 6 1 3 SYN > /dev/tap0 || exit 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 6 3 1 SYN > /dev/tap0 || exit 1

if wait %../tools/rcv_ip; then :
else
    echo rcv_ip failed for tcp --dports 1:
    cat $TMPFILE
    exit 1
fi
if [ "`cat $TMPFILE`" != "$TAP0NET.2 $TAP1NET.2 100 6 3 1 SYN " ]
then
    echo rcv_ip wrong for tcp --dports 1:
    cat $TMPFILE
    exit 1
fi

# Finally, multiple ports tests.
# Test --ports
../packet-filter/userspace/iptables -R FORWARD 1 -m multiport -p udp --ports 1,2,3,4,5,6,7 -j ACCEPT || exit 1

../tools/rcv_ip 10 63 < /dev/tap1 > $TMPFILE &
sleep 1

for sport in 8 7 6 5 4 3 2 1; do
for dport in 8 7 6 5 4 3 2 1; do
    ../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 17 $sport $dport > /dev/tap0 || exit 1
done
done

if wait %../tools/rcv_ip; then :
else
    echo rcv_ip failed for multiple udp --ports:
    cat $TMPFILE
    exit 1
fi
if grep '8 8' < $TMPFILE
then
    echo rcv_ip wrong for multiple udp --ports:
    cat $TMPFILE
    exit 1
fi

../packet-filter/userspace/iptables -R FORWARD 1 -m multiport -p tcp --ports 1,2,3,4,5,6,7 -j ACCEPT || exit 1

../tools/rcv_ip 10 63 < /dev/tap1 > $TMPFILE &
sleep 1

for sport in 8 7 6 5 4 3 2 1; do
for dport in 8 7 6 5 4 3 2 1; do
    ../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 6 $sport $dport SYN > /dev/tap0 || exit 1
done
done

if wait %../tools/rcv_ip; then :
else
    echo rcv_ip failed for multiple tcp --ports:
    cat $TMPFILE
    exit 1
fi

if grep '8 8 SYN' < $TMPFILE
then
    echo rcv_ip wrong for multiple tcp --ports:
    cat $TMPFILE
    exit 1
fi

# Now we test --sports
../packet-filter/userspace/iptables -R FORWARD 1 -m multiport -p udp --sports 1,2,3,4,5,6,7 -j ACCEPT || exit 1

../tools/rcv_ip 2 7 < /dev/tap1 > $TMPFILE &
sleep 1

for sport in 8 7 6 5 4 3 2 1; do
    ../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 17 $sport 1 > /dev/tap0 || exit 1
done

if wait %../tools/rcv_ip; then :
else
    echo rcv_ip failed for multiple udp --sports:
    cat $TMPFILE
    exit 1
fi
if [ "`cat $TMPFILE`" != "$TAP0NET.2 $TAP1NET.2 100 17 7 1 
$TAP0NET.2 $TAP1NET.2 100 17 6 1 
$TAP0NET.2 $TAP1NET.2 100 17 5 1 
$TAP0NET.2 $TAP1NET.2 100 17 4 1 
$TAP0NET.2 $TAP1NET.2 100 17 3 1 
$TAP0NET.2 $TAP1NET.2 100 17 2 1 
$TAP0NET.2 $TAP1NET.2 100 17 1 1 " ]
then
    echo rcv_ip wrong for multiple udp --sports:
    cat $TMPFILE
    exit 1
fi

../packet-filter/userspace/iptables -R FORWARD 1 -m multiport -p tcp --sports 1,2,3,4,5,6,7 -j ACCEPT || exit 1

../tools/rcv_ip 2 7 < /dev/tap1 > $TMPFILE &
sleep 1

for sport in 8 7 6 5 4 3 2 1; do
    ../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 6 $sport 1 SYN > /dev/tap0 || exit 1
done

if wait %../tools/rcv_ip; then :
else
    echo rcv_ip failed for multiple tcp --sports:
    cat $TMPFILE
    exit 1
fi
if [ "`cat $TMPFILE`" != "$TAP0NET.2 $TAP1NET.2 100 6 7 1 SYN 
$TAP0NET.2 $TAP1NET.2 100 6 6 1 SYN 
$TAP0NET.2 $TAP1NET.2 100 6 5 1 SYN 
$TAP0NET.2 $TAP1NET.2 100 6 4 1 SYN 
$TAP0NET.2 $TAP1NET.2 100 6 3 1 SYN 
$TAP0NET.2 $TAP1NET.2 100 6 2 1 SYN 
$TAP0NET.2 $TAP1NET.2 100 6 1 1 SYN " ]
then
    echo rcv_ip wrong for multiple tcp --sports:
    cat $TMPFILE
    exit 1
fi

# Now we test --dports
../packet-filter/userspace/iptables -R FORWARD 1 -m multiport -p udp --dports 1,2,3,4,5,6,7 -j ACCEPT || exit 1

../tools/rcv_ip 2 7 < /dev/tap1 > $TMPFILE &
sleep 1

for dport in 8 7 6 5 4 3 2 1; do
    ../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 17 1 $dport > /dev/tap0 || exit 1
done

if wait %../tools/rcv_ip; then :
else
    echo rcv_ip failed for multiple udp --dports:
    cat $TMPFILE
    exit 1
fi
if [ "`cat $TMPFILE`" != "$TAP0NET.2 $TAP1NET.2 100 17 1 7 
$TAP0NET.2 $TAP1NET.2 100 17 1 6 
$TAP0NET.2 $TAP1NET.2 100 17 1 5 
$TAP0NET.2 $TAP1NET.2 100 17 1 4 
$TAP0NET.2 $TAP1NET.2 100 17 1 3 
$TAP0NET.2 $TAP1NET.2 100 17 1 2 
$TAP0NET.2 $TAP1NET.2 100 17 1 1 " ]
then
    echo rcv_ip wrong for multiple udp --dports:
    cat $TMPFILE
    exit 1
fi

../packet-filter/userspace/iptables -R FORWARD 1 -m multiport -p tcp --dports 1,2,3,4,5,6,7 -j ACCEPT || exit 1

../tools/rcv_ip 2 7 < /dev/tap1 > $TMPFILE &
sleep 1

for dport in 8 7 6 5 4 3 2 1; do
    ../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 6 1 $dport SYN > /dev/tap0 || exit 1
done

if wait %../tools/rcv_ip; then :
else
    echo rcv_ip failed for multiple tcp --sports:
    cat $TMPFILE
    exit 1
fi
if [ "`cat $TMPFILE`" != "$TAP0NET.2 $TAP1NET.2 100 6 1 7 SYN 
$TAP0NET.2 $TAP1NET.2 100 6 1 6 SYN 
$TAP0NET.2 $TAP1NET.2 100 6 1 5 SYN 
$TAP0NET.2 $TAP1NET.2 100 6 1 4 SYN 
$TAP0NET.2 $TAP1NET.2 100 6 1 3 SYN 
$TAP0NET.2 $TAP1NET.2 100 6 1 2 SYN 
$TAP0NET.2 $TAP1NET.2 100 6 1 1 SYN " ]
then
    echo rcv_ip wrong for multiple tcp --dports:
    cat $TMPFILE
    exit 1
fi

../packet-filter/userspace/iptables -F FORWARD || exit 1
rmmod ipt_multiport || exit 1
exit 0
