#! /bin/sh

# Test that packets set up a connection, and same packets don't set up
# new ones.

../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 1 8 0 55 57 > /dev/tap0
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 6 1024 1025 SYN > /dev/tap0
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 17 1024 1025 > /dev/tap0
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 2 > /dev/tap0

dd bs=1k if=/proc/net/ip_conntrack > $TMPFILE 2>/dev/null

# ICMP
if grep "^icmp     1 [0-9]* src=$TAP0NET.2 dst=$TAP1NET.2 type=8 code=0 id=55 \[UNREPLIED\] src=$TAP1NET.2 dst=$TAP0NET.2 type=0 code=0 id=55 $" $TMPFILE > /dev/null
then
    :
else
    echo ICMP not tracked properly:
    grep "^icmp" $TMPFILE
    exit 1
fi

# UDP
if grep "^udp      17 [0-9]* src=$TAP0NET.2 dst=$TAP1NET.2 sport=1024 dport=1025 \[UNREPLIED\] src=$TAP1NET.2 dst=$TAP0NET.2 sport=1025 dport=1024 $" $TMPFILE > /dev/null
then
    :
else
    echo UDP not tracked properly:
    grep "^udp" $TMPFILE
    exit 1
fi

# TCP
if grep "^tcp      6 [0-9]* SYN_SENT src=$TAP0NET.2 dst=$TAP1NET.2 sport=1024 dport=1025 \[UNREPLIED\] src=$TAP1NET.2 dst=$TAP0NET.2 sport=1025 dport=1024 $" $TMPFILE > /dev/null
then
    :
else
    echo TCP not tracked properly:
    grep "^tcp" $TMPFILE
    exit 1
fi

# Unknown
if grep "^unknown  2 [0-9]* src=$TAP0NET.2 dst=$TAP1NET.2 \[UNREPLIED\] src=$TAP1NET.2 dst=$TAP0NET.2 $" $TMPFILE > /dev/null
then
    :
else
    echo Unknown not tracked properly:
    grep "^unknown " $TMPFILE
    exit 1
fi

# Same packets again: should be the same (except icmp special case)
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 6 1024 1025 SYN > /dev/tap1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 6 1024 1025 SYN > /dev/tap1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 17 1024 1025 > /dev/tap1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 17 1024 1025 > /dev/tap1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 2 > /dev/tap1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 2 > /dev/tap1

if [ `wc -l < /proc/net/ip_conntrack` != 4 ]
then
    echo Same packets caused new connections:
    cat /proc/net/ip_conntrack
    exit 1
fi

# New packets: new connections.
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 1 8 0 56 58 > /dev/tap1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 6 1024 1026 SYN > /dev/tap1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 6 1023 1025 SYN > /dev/tap1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 17 1024 1026 > /dev/tap1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 17 1023 1025 > /dev/tap1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 3 > /dev/tap1

if [ `wc -l < /proc/net/ip_conntrack` != 10 ]
then
    echo New packets didn\'t cause new connections:
    cat /proc/net/ip_conntrack
    exit 1
fi

exit 0
