#! /bin/sh

# Test that ICMP replies don't get mangled when no NAT in place.

MYTMPFILE=`tempfile`

# Need job control.
set -m

# Test UDP ports across 2 src and dest IP addresses, in order.
for src in 2 3
do
    for dst in 2 3
    do
	for sp in 1023 1024
	do 
	    for dp in 1023 1024
	    do
		for err in 3 4 11 12
		do
		    # Put raw packet into MYTMPFILE.
		    dd bs=116 if=/dev/tap0 of=$MYTMPFILE count=1 2>/dev/null & 
		    sleep 1
		    ../tools/gen_ip $TAP1NET.$src $TAP0NET.$dst 100 17 $sp $dp > /dev/tap1
		    wait

		    # Put raw error reply packet into TMPFILE
		    dd bs=72 if=/dev/tap1 of=$TMPFILE count=1 2>/dev/null &
		    sleep 1

		    # Generate ICMP error reply from raw packet.
		    ../tools/gen_err $TAP0NET.$dst $err 0 < $MYTMPFILE > /dev/tap0
		    rm $MYTMPFILE

		    wait

		    # Check reply:
		    ERRREPLY=`../tools/rcv_ip 1 1 < $TMPFILE`
		    if [ "$ERRREPLY" != "$TAP0NET.$dst $TAP1NET.$src 56 1 $err 0 " ]
		    then
			echo Unexpected reply: "$ERRREPLY" vs "$TAP0NET.$dst $TAP1NET.$src 56 1 $err 0 "
			exit 1
		    fi

		    # Check packet inside reply:
		    # obs avoids nagle chopping traffic through pipe
		    INTERNAL=`dd ibs=1 skip=28 obs=100 if=$TMPFILE 2>/dev/null | ../tools/rcv_ip 1 1`
		    if [ "$INTERNAL" != "$TAP1NET.$src $TAP0NET.$dst 100 17 $sp $dp -TRUNCATED- " ]
		    then
			echo Unexpected internal packet: for "$TAP1NET.$src $TAP0NET.$dst 100 17 $sp $dp": $INTERNAL
			exit 1
		    fi
		done
	    done
        done
    done
done
