#! /bin/sh

# Mapping everything from 192.168.1.* to src of one of our interfaces
# should not effect connections which are actually to one of those
# interfaces (bug 01).

# We can test this by generating a SYN to that interface.

# Need job control.

set -m

# Map TAP1 net sources to TAP0.
../NAT/userspace/ipnatctl -I -s $TAP1NET.0/24 -b source -t $TAP0 || exit 1

# Test sending SYN to TAP0; rely on no anti-spoofing.
../tools/rcv_ip 2 1 < /dev/tap1 > $TMPFILE &
sleep 1

../tools/gen_ip $TAP1NET.5 $TAP0 40 6 100 1 SYN > /dev/tap0 || exit 1

if wait %../tools/rcv_ip; then :
else
    echo ../tools/rcv_ip failed:
    cat $TMPFILE
    cat /proc/net/ip_conntrack
    exit 1
fi

# ACK will cover data + 1.
if [ "`cat $TMPFILE`" != "$TAP0 $TAP1NET.5 40 6 1 100 RST ACK=1 " ]
then
    echo RST reply failed
    cat $TMPFILE
    cat /proc/net/ip_conntrack
    exit 1
fi

insmod ../NAT/mapping-types/ip_nat_map_masquerade.o || exit 1

# Same test with masquerade instead.
../NAT/userspace/ipnatctl -D -s $TAP1NET.0/24 -b source -t $TAP0 || exit 1
../NAT/userspace/ipnatctl -I -s $TAP1NET.0/24 -b source -m masquerade || exit 1

# Test sending SYN to TAP0; rely on no anti-spoofing.
../tools/rcv_ip 2 1 < /dev/tap1 > $TMPFILE &
sleep 1

../tools/gen_ip $TAP1NET.5 $TAP0 40 6 100 1 SYN > /dev/tap0 || exit 1

if wait %../tools/rcv_ip; then :
else
    echo ../tools/rcv_ip failed with masquerade:
    cat $TMPFILE
    exit 1
fi

if [ "`cat $TMPFILE`" != "$TAP0 $TAP1NET.5 40 6 1 100 RST ACK=1 " ]
then
    echo RST reply failed with masquerade
    cat $TMPFILE
    cat /proc/net/ip_conntrack
    exit 1
fi

# Now, try a local packet to that interface.
ping -c 1 $TAP1 || exit 1

../NAT/userspace/ipnatctl -D -s $TAP1NET.0/24 -b source -m masquerade || exit 1
rmmod ip_nat_map_masquerade || exit 1

exit 0
