#! /bin/sh

# Test interface matching.

# Need job control.

set -m

# These should fail:
../NAT/userspace/ipnatctl -I -s $TAP1NET.0/24 -b source -i tap0 -t $TAP0 2>/dev/null && exit 1
../NAT/userspace/ipnatctl -I -s $TAP1NET.0/24 -b dest -o tap0 -t $TAP0 2>/dev/null && exit 1
../NAT/userspace/ipnatctl -I -s $TAP1NET.0/24 -i tap0 -b source -t $TAP0 2>/dev/null && exit 1
../NAT/userspace/ipnatctl -I -s $TAP1NET.0/24 -o tap0 -b dest-t $TAP0 2>/dev/null && exit 1

# These should succeed:
../NAT/userspace/ipnatctl -I -o tap1 -b source -t $TAP1 || exit 1
../NAT/userspace/ipnatctl -D -o tap1 -b source -t $TAP1 || exit 1
../NAT/userspace/ipnatctl -I -b source -o tap1 -t $TAP1 || exit 1

# Send ping, check mapping:
../tools/rcv_ip 2 1 < /dev/tap1 > $TMPFILE &
sleep 1

../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 1 8 0 55 57 > /dev/tap0 || exit 1

if wait %../tools/rcv_ip; then :
else
    echo ../tools/rcv_ip failed:
    cat $TMPFILE
    cat /proc/net/ip_conntrack
    exit 1
fi

if [ "`cat $TMPFILE`" != "$TAP1 $TAP1NET.2 100 1 8 0 55 57 " ]
then
    echo SRC mapping failed
    cat $TMPFILE
    cat /proc/net/ip_conntrack
    exit 1
fi

# Send ping, check no mapping:
../tools/rcv_ip 2 1 < /dev/tap0 > $TMPFILE &
sleep 1

../tools/gen_ip $TAP1NET.2 $TAP0NET.2 100 1 8 0 55 57 > /dev/tap1 || exit 1

if wait %../tools/rcv_ip; then :
else
    echo ../tools/rcv_ip failed on tap0:
    cat $TMPFILE
    cat /proc/net/ip_conntrack
    exit 1
fi

if [ "`cat $TMPFILE`" != "$TAP1NET.2 $TAP0NET.2 100 1 8 0 55 57 " ]
then
    echo SRC mapping wierd on tap0
    cat $TMPFILE
    cat /proc/net/ip_conntrack
    exit 1
fi

# Wildcard check:
../NAT/userspace/ipnatctl -D -o tap1 -b source -t $TAP1 || exit 1
../NAT/userspace/ipnatctl -I -b source -o tap1+ -t $TAP1 || exit 1

# Send ping, check mapping:
../tools/rcv_ip 2 1 < /dev/tap1 > $TMPFILE &
sleep 1

../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 1 8 0 55 57 > /dev/tap0 || exit 1

if wait %../tools/rcv_ip; then :
else
    echo ../tools/rcv_ip wildcard failed:
    cat $TMPFILE
    cat /proc/net/ip_conntrack
    exit 1
fi

if [ "`cat $TMPFILE`" != "$TAP1 $TAP1NET.2 100 1 8 0 55 57 " ]
then
    echo SRC mapping wildcard failed
    cat $TMPFILE
    cat /proc/net/ip_conntrack
    exit 1
fi

# Send ping, check no mapping:
../tools/rcv_ip 2 1 < /dev/tap0 > $TMPFILE &
sleep 1

../tools/gen_ip $TAP1NET.2 $TAP0NET.2 100 1 8 0 55 57 > /dev/tap1 || exit 1

if wait %../tools/rcv_ip; then :
else
    echo ../tools/rcv_ip wildcard failed on tap0:
    cat $TMPFILE
    cat /proc/net/ip_conntrack
    exit 1
fi

if [ "`cat $TMPFILE`" != "$TAP1NET.2 $TAP0NET.2 100 1 8 0 55 57 " ]
then
    echo SRC mapping wildcard wierd on tap0
    cat $TMPFILE
    cat /proc/net/ip_conntrack
    exit 1
fi

# Real wildcard check:
../NAT/userspace/ipnatctl -D -o tap1+ -b source -t $TAP1 || exit 1
../NAT/userspace/ipnatctl -I -b source -o tap+ -t $TAP1 || exit 1

# Send ping, check mapping:
../tools/rcv_ip 2 1 < /dev/tap1 > $TMPFILE &
sleep 1

../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 1 8 0 55 57 > /dev/tap0 || exit 1

if wait %../tools/rcv_ip; then :
else
    echo ../tools/rcv_ip wildcard2 failed:
    cat $TMPFILE
    cat /proc/net/ip_conntrack
    exit 1
fi

if [ "`cat $TMPFILE`" != "$TAP1 $TAP1NET.2 100 1 8 0 55 57 " ]
then
    echo SRC mapping wildcard2 failed
    cat $TMPFILE
    cat /proc/net/ip_conntrack
    exit 1
fi

# Send ping, check mapping:
../tools/rcv_ip 2 1 < /dev/tap0 > $TMPFILE &
sleep 1

../tools/gen_ip $TAP1NET.2 $TAP0NET.2 100 1 8 0 55 57 > /dev/tap1 || exit 1

if wait %../tools/rcv_ip; then :
else
    echo ../tools/rcv_ip wildcard2 failed on tap0:
    cat $TMPFILE
    cat /proc/net/ip_conntrack
    exit 1
fi

if [ "`cat $TMPFILE`" != "$TAP1 $TAP0NET.2 100 1 8 0 55 57 " ]
then
    echo SRC mapping wildcard2 wierd on tap0
    echo \`"`cat $TMPFILE`"\' vs \`"$TAP1 $TAP0NET.2 100 1 8 0 55 57 "\'
    cat /proc/net/ip_conntrack
    exit 1
fi

exit 0
