#! /bin/sh

# Test that NAT fails gracefully when conntrack deleted underneath it.

# Need job control.
set -m

# Source manipulation: conntrack detected on PRE_ROUTING, manip to be done
# on POST_ROUTING.
../NAT/userspace/ipnatctl -I -s $TAP0NET.0/24 -d $TAP1NET.2 -b source -t $TAP1NET.3 || exit 1

# Normal mapping without intercept should be OK.
../tools/rcv_ip 2 1 < /dev/tap1 > $TMPFILE &
sleep 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 17 1024 53 > /dev/tap0

if wait %../tools/rcv_ip; then :
else
    echo Normal rcv_ip failed.
    cat $TMPFILE
    exit 1
fi

if [ "`cat $TMPFILE`" != "$TAP1NET.3 $TAP1NET.2 100 17 1024 53 " ]
then
    echo Normal UDP mapping failed.
    cat $TMPFILE
    exit 1
fi

# Intercept in forward hook.
insmod ../tools/redirect.o hook=2 || exit 1
insmod ../device/netfilter_dev.o || exit 1

../tools/intercept FORWARD ACCEPT 65 1 61 > /dev/null &
sleep 1

# UDP will get held up for 61 seconds
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 17 1024 53 > /dev/tap0

echo Please wait 60 seconds...
# Now, when intercept reinjects, conntrack will be gone => no mapping.
if ../tools/rcv_ip 65 1 < /dev/tap1 > $TMPFILE
then
    :
else
    echo Failed to receive packet.
    exit 1
fi

if wait %../tools/intercept; then :
else
    echo ../tools/intercept failed.
    exit 1
fi

if [ "`cat $TMPFILE`" != "$TAP0NET.2 $TAP1NET.2 100 17 1024 53 " ]
then
    echo Unexpected output:
    cat $TMPFILE
    exit 1
fi

# Delete modules
rmmod redirect || exit 1
rmmod netfilter_dev || exit 1

exit 0

