#! /bin/sh

# Simple script to test that inserting & deleting rules and chains works.

# Need job control.
set -m

ipchains -N foo || exit 1
ipchains -N foo 2>/dev/null && exit 1

ipchains -X foo || exit 1
ipchains -X foo 2>/dev/null && exit 1

# Can't delete builtins.

ipchains -X input 2>/dev/null && exit 1
ipchains -X forward 2>/dev/null && exit 1
ipchains -X output 2>/dev/null && exit 1
ipchains -X RETURN 2>/dev/null && exit 1
ipchains -X MASQ 2>/dev/null && exit 1
ipchains -X REDIRECT 2>/dev/null && exit 1
ipchains -X ACCEPT 2>/dev/null && exit 1
ipchains -X DENY 2>/dev/null && exit 1
ipchains -X REJECT 2>/dev/null && exit 1

# FIXME: Don't let them create RETURN, MASQ, etc.
# Can't create builtins.
ipchains -N input 2>/dev/null && exit 1
ipchains -N forward 2>/dev/null && exit 1
ipchains -N output 2>/dev/null && exit 1
#ipchains -N RETURN 2>/dev/null && exit 1
#ipchains -N MASQ 2>/dev/null && exit 1
#ipchains -N REDIRECT 2>/dev/null && exit 1
#ipchains -N ACCEPT 2>/dev/null && exit 1
#ipchains -N DENY 2>/dev/null && exit 1
#ipchains -N REJECT 2>/dev/null && exit 1

# Insert test.
ipchains -N foo || exit 1
ipchains -A foo || exit 1

# Can't delete non-empty.
ipchains -X foo 2>/dev/null && exit 1

# Delete tests.
ipchains -D foo || exit 1
ipchains -A foo || exit 1
ipchains -D foo 1 || exit 1

# Can delete empty.
ipchains -X foo || exit 1

# Can't delete referred-to.
ipchains -N foo || exit 1
ipchains -N foo2 || exit 1
ipchains -A foo -j foo2 || exit 1
ipchains -X foo2 2>/dev/null && exit 1

# Can't delete non-empty referred-to.
ipchains -A foo2 || exit 1
ipchains -X foo2 2>/dev/null && exit 1

# Clear reference and check.
ipchains -D foo 1 || exit 1
ipchains -X foo2 2>/dev/null && exit 1

# Clear chain and delete.
ipchains -D foo2 1 || exit 1
ipchains -X foo2 || exit 1

# Test rule append.
ipchains -A foo -j DENY || exit 1
ipchains -A foo -j REJECT || exit 1
ipchains -A foo -j ACCEPT || exit 1

# Test list of all.
if [ "`ipchains -L`" != "Chain input (policy ACCEPT):
Chain forward (policy ACCEPT):
Chain output (policy ACCEPT):
Chain foo (0 references):
target     prot opt     source                destination           ports
DENY       all  ------  anywhere             anywhere              n/a
REJECT     all  ------  anywhere             anywhere              n/a
ACCEPT     all  ------  anywhere             anywhere              n/a" ]
then
    echo Unexpected ipchains -L output:
    ipchains -L
    exit 1
fi

# Test list of foo only.
if [ "`ipchains -L foo`" != "Chain foo (0 references):
target     prot opt     source                destination           ports
DENY       all  ------  anywhere             anywhere              n/a
REJECT     all  ------  anywhere             anywhere              n/a
ACCEPT     all  ------  anywhere             anywhere              n/a" ]
then
    echo Unexpected ipchains -L foo output.
    ipchains -L foo
    exit 1
fi

# Test insert.
ipchains -I foo -j RETURN || exit 1
if [ "`ipchains -L foo`" != "Chain foo (0 references):
target     prot opt     source                destination           ports
RETURN     all  ------  anywhere             anywhere              n/a
DENY       all  ------  anywhere             anywhere              n/a
REJECT     all  ------  anywhere             anywhere              n/a
ACCEPT     all  ------  anywhere             anywhere              n/a" ]
then
    echo Unexpected ipchains -L foo output after RETURN.
    ipchains -L foo
    exit 1
fi

ipchains -I foo 1 -j MASQ || exit 1
if [ "`ipchains -L foo`" != "Chain foo (0 references):
target     prot opt     source                destination           ports
MASQ       all  ------  anywhere             anywhere              n/a
RETURN     all  ------  anywhere             anywhere              n/a
DENY       all  ------  anywhere             anywhere              n/a
REJECT     all  ------  anywhere             anywhere              n/a
ACCEPT     all  ------  anywhere             anywhere              n/a" ]
then
    echo Unexpected ipchains -L foo output after MASQ
    ipchains -L foo
    exit 1
fi

# Insert past end must fail.
ipchains -I foo 7 2>/dev/null && exit 1

ipchains -I foo 6 -p tcp -j REDIRECT || exit 1

if [ "`ipchains -L foo`" != "Chain foo (0 references):
target     prot opt     source                destination           ports
MASQ       all  ------  anywhere             anywhere              n/a
RETURN     all  ------  anywhere             anywhere              n/a
DENY       all  ------  anywhere             anywhere              n/a
REJECT     all  ------  anywhere             anywhere              n/a
ACCEPT     all  ------  anywhere             anywhere              n/a
REDIRECT   tcp  ------  anywhere             anywhere              any ->   any =>  any" ]
then
    echo Unexpected ipchains -L foo output after REDIRECT
    ipchains -L foo
    exit 1
fi

ipchains -I foo 2 || exit 1

if [ "`ipchains -L foo`" != "Chain foo (0 references):
target     prot opt     source                destination           ports
MASQ       all  ------  anywhere             anywhere              n/a
-          all  ------  anywhere             anywhere              n/a
RETURN     all  ------  anywhere             anywhere              n/a
DENY       all  ------  anywhere             anywhere              n/a
REJECT     all  ------  anywhere             anywhere              n/a
ACCEPT     all  ------  anywhere             anywhere              n/a
REDIRECT   tcp  ------  anywhere             anywhere              any ->   any =>  any" ]
then
    echo Unexpected ipchains -L foo output after -I 2
    ipchains -L foo
    exit 1
fi

# Deletion tests.
ipchains -D foo 8 2>/dev/null && exit 1
ipchains -D foo 7 || exit 1
if [ "`ipchains -L foo`" != "Chain foo (0 references):
target     prot opt     source                destination           ports
MASQ       all  ------  anywhere             anywhere              n/a
-          all  ------  anywhere             anywhere              n/a
RETURN     all  ------  anywhere             anywhere              n/a
DENY       all  ------  anywhere             anywhere              n/a
REJECT     all  ------  anywhere             anywhere              n/a
ACCEPT     all  ------  anywhere             anywhere              n/a" ]
then
    echo Unexpected ipchains -L foo output after -D 7
    ipchains -L foo
    exit 1
fi

ipchains -D foo 5 || exit 1
if [ "`ipchains -L foo`" != "Chain foo (0 references):
target     prot opt     source                destination           ports
MASQ       all  ------  anywhere             anywhere              n/a
-          all  ------  anywhere             anywhere              n/a
RETURN     all  ------  anywhere             anywhere              n/a
DENY       all  ------  anywhere             anywhere              n/a
ACCEPT     all  ------  anywhere             anywhere              n/a" ]
then
    echo Unexpected ipchains -L foo output after -D 5
    ipchains -L foo
    exit 1
fi

ipchains -D foo -j RETURN || exit 1

if [ "`ipchains -L foo`" != "Chain foo (0 references):
target     prot opt     source                destination           ports
MASQ       all  ------  anywhere             anywhere              n/a
-          all  ------  anywhere             anywhere              n/a
DENY       all  ------  anywhere             anywhere              n/a
ACCEPT     all  ------  anywhere             anywhere              n/a" ]
then
    echo Unexpected ipchains -L foo output after -D RETURN
    ipchains -L foo
    exit 1
fi

# Test flush.
ipchains -F foo || exit 1
if [ "`ipchains -L foo`" != "Chain foo (0 references):" ]
then
    echo Unexpected ipchains -L foo output after -F
    ipchains -L foo
    exit 1
fi

ipchains -F foo || exit 1
if [ "`ipchains -L foo`" != "Chain foo (0 references):" ]
then
    echo Unexpected ipchains -L foo output after second -F
    ipchains -L foo
    exit 1
fi

# Delete chain.
ipchains -X foo || exit 1
ipchains -F foo 2>/dev/null && exit 1

# Replace tests.
ipchains -N foo || exit 1
ipchains -R foo 1 2>/dev/null && exit 1
ipchains -A foo || exit 1
ipchains -R foo 1 -j DENY || exit 1

if [ "`ipchains -L foo`" != "Chain foo (0 references):
target     prot opt     source                destination           ports
DENY       all  ------  anywhere             anywhere              n/a" ]
then
    echo Unexpected ipchains -L foo output after -R 1
    ipchains -L foo
    exit 1
fi

# Test counter zeroing.
ipchains -A forward -p icmp -j RETURN || exit 1
../tools/rcv_ip 2 1 < /dev/tap1 > $TMPFILE &
sleep 1

../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 1 8 0 55 57 > /dev/tap0

if wait %../tools/rcv_ip; then :
else
    echo ../tools/rcv_ip failed:
    cat $TMPFILE
    exit 1
fi

if [ "`ipchains -L forward -v`" != "Chain forward (policy ACCEPT: 1 packets, 100 bytes):
 pkts bytes target     prot opt    tosa tosx  ifname     mark       outsize  source                destination           ports
    1   100 RETURN     icmp ------ 0xFF 0x00  any                            anywhere             anywhere              any ->   any" ]
then
    echo Unexpected ipchains -L forward counter test.
    ipchains -L forward -v
    exit 1
fi

ipchains -Z forward || exit 1
if [ "`ipchains -L forward -v`" != "Chain forward (policy ACCEPT: 1 packets, 100 bytes):
 pkts bytes target     prot opt    tosa tosx  ifname     mark       outsize  source                destination           ports
    0     0 RETURN     icmp ------ 0xFF 0x00  any                            anywhere             anywhere              any ->   any" ]
then
    echo Unexpected ipchains -L forward after -Z forward.
    ipchains -L forward
    exit 1
fi

../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 1 8 0 55 57 > /dev/tap0
ipchains -Z || exit 1
if [ "`ipchains -L forward -v`" != "Chain forward (policy ACCEPT: 2 packets, 200 bytes):
 pkts bytes target     prot opt    tosa tosx  ifname     mark       outsize  source                destination           ports
    0     0 RETURN     icmp ------ 0xFF 0x00  any                            anywhere             anywhere              any ->   any" ]
then
    echo Unexpected ipchains -L forward after -Z.
    ipchains -L forward -v
    exit 1
fi

# Policy setting.
ipchains -F forward || exit 1
ipchains -P forward DENY || exit 1
../tools/rcv_ip 2 1 < /dev/tap1 > $TMPFILE &
sleep 1

../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 1 8 0 55 57 > /dev/tap0

if wait %../tools/rcv_ip
then 
    echo Policy setting failed.
    cat $TMPFILE
    exit 1
fi

if [ "`ipchains -L forward -v`" != "Chain forward (policy DENY: 3 packets, 300 bytes):" ]
then
    echo Unexpected ipchains -L forward after -P.
    ipchains -L forward -v
    exit 1
fi

exit 0
