#! /bin/sh
# Shell script to run test suite.

# fix the locale to ensure that ifconfig returns a line with 'inet addr:'
# foreign testers can have trouble with locale, especialy french ones :)
LANG=C; export LANG

TAP0=192.168.1.1
TAP1=192.168.2.1
TAP0NET=192.168.1
TAP1NET=192.168.2

if [ x"$1" = x"-v" ]; then VERBOSE=1; shift; fi
if [ x"$1" = x"-p" ]
then
    echo Hi Rusty.  Fucked up, huh\?  Battening down the hatches.
    mount -o remount -r /home
    mount -o remount -r /usr
    shift
fi

TMPFILE=`tempfile`

setup()
{
    # Devices /dev/tap0 and /dev/tap1 must exist.
    if [ ! -c /dev/tap0 ]
    then
	echo Creating /dev/tap0.
	mknod /dev/tap0 c 36 16
    fi

    if [ ! -c /dev/tap1 ]
    then
	echo Creating /dev/tap1.
	mknod /dev/tap1 c 36 17
    fi

    # Modules for them must exist.
    if grep "tap0" < /proc/modules > /dev/null; then :
    else
	echo Inserting ethertap module for tap0.
	insmod ethertap -o tap0 unit=0
    fi

    if grep "tap1" < /proc/modules > /dev/null; then :
    else
	echo Inserting ethertap module for tap1.
	insmod ethertap -o tap1 unit=1
    fi

    # Set up interfaces if not already.
    if ifconfig | grep tap0 > /dev/null
    then
	IPADDR=`ifconfig tap0 | grep 'inet addr:' | cut -d: -f2 | cut -d\  -f1`
	if [ $IPADDR != $TAP0 ]
	then
	    echo Adjusting tap0 interface.
	    ifconfig tap0 down
	    ifconfig tap0 $TAP0
        fi
    else
	echo Bringing tap0 interface up.
	ifconfig tap0 $TAP0
    fi

    if ifconfig | grep tap1 > /dev/null
    then
	IPADDR=`ifconfig tap1 | grep 'inet addr:' | cut -d: -f2 | cut -d\  -f1`
	if [ $IPADDR != $TAP1 ]
	then
	    echo Adjusting tap1 interface.
	    ifconfig tap1 down
	    ifconfig tap1 $TAP1
        fi
    else
	echo Bringing tap1 interface up.
	ifconfig tap1 $TAP1
    fi

    # Turn off anti-spoof on those interfaces.
    echo 0 > /proc/sys/net/ipv4/conf/tap0/rp_filter
    echo 0 > /proc/sys/net/ipv4/conf/tap1/rp_filter

    # Load netlink device if it's a module.
    modprobe netlink_dev.o

    ../packet-filter/userspace/iptables -F 2>/dev/null
    # Delete the various modules.
    for module in drop_invalid ip_nat_map_masquerade ip_nat_map_redirect ip_nat_map_static ip_nat_ftp ip_nat ipt_state ipt_multiport ipt_LOG ipt_limit ipt_REJECT ipt_QUEUE ipt_icmp ipt_udp ipt_tcp ipt_mac ipt_unclean iptables ip_conntrack_ftp ip_conntrack redirect netfilter_dev ipchains ipfwadm
    do
	rmmod $module 2>/dev/null
	if lsmod | grep "^$module " > /dev/null
	then
	    echo Can\'t remove $module.
	    exit 1
	fi
    done

    # enable forwarding
    echo 1 > /proc/sys/net/ipv4/ip_forward
}

logs_check()
{
   if dmesg | egrep '^(ASSERT|ip_local_deliver|ip_dev_loopback_xmit|ip_finish_output)' >/dev/null
   then
       echo Assertion from test ${1}:
       dmesg | egrep '^(ASSERT|ip_local_deliver|ip_dev_loopback_xmit|ip_finish_output)'
       exit 1
   fi
   dmesg -c >/dev/null
}

cleanup()
{
    rm -f $TMPFILE
    exit $1
}

list_dirs()
{
    if [ -n "$1" ]
    then
	ls */test.sh | tail +`ls */test.sh | fgrep -n $1/test.sh | cut -d: -f1`
    else
	ls */test.sh
    fi
}

# First do setup stuff.
setup

# Want job control
set -m

# Now, simple ping test.
../tools/rcv_ip 2 1 < /dev/tap1 > $TMPFILE &
sleep 1
../tools/gen_ip $TAP0NET.2 $TAP1NET.2 100 1 8 0 1 2 > /dev/tap0

if wait %../tools/rcv_ip; then :
else
    echo Ping test failed `cat $TMPFILE`.
    cleanup 1
fi

../tools/rcv_ip 2 1 < /dev/tap0 > $TMPFILE &
sleep 1
../tools/gen_ip $TAP1NET.2 $TAP0NET.2 100 1 0 0 1 2 > /dev/tap1

if wait %../tools/rcv_ip; then :
else
    echo Pong test failed `cat $TMPFILE`.
    cleanup 1
fi

export TMPFILE TAP0 TAP1 TAP0NET TAP1NET VERBOSE
export -f logs_check

echo Setup complete.
START_DIR=`echo $1 | cut -d/ -f1`
START_FILE=$1
DIRS=`list_dirs $START_DIR`

for f in $DIRS
do
    [ -z "$VERBOSE" ] || echo Running tests for `dirname $f`.
    dmesg -c >/dev/null
    if $f $START_FILE
    then
	echo Tests for `dirname $f` succeeded.
    else
	echo Test for `dirname $f` failed.
	cleanup 1
    fi
    START_FILE=""
done

cleanup 0


