/* Simple program to raw send ip packet given on stdin.  */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <netdb.h>
#include <netinet/ip.h>
#if defined(__GLIBC__) && __GLIBC__ == 2
#include <netinet/udp.h>
#include <netinet/tcp.h>
#include <netinet/ip_icmp.h>
#include <net/ethernet.h>
#else
#include <linux/tcp.h>
#include <linux/udp.h>
#include <linux/icmp.h>
#include <linux/if_ether.h>
#endif
#include <sys/types.h>

struct packet {
	u_int16_t pad;
#if defined(__GLIBC__) && __GLIBC__ == 2
	struct ether_header ehdr;
#else
	struct ethhdr ehdr;
#endif
	struct iphdr iph;
	union {
		struct tcphdr tcph;
		struct udphdr udph;
		struct icmphdr icmph;
		unsigned char raw[0];
	} u;
	unsigned char data[65535];
};

int main(int argc, char *argv[])
{
	struct packet packet;
	int rval, rawsock;
	int set = 1;

	if (argc != 1) {
		fprintf(stderr, "local_ip: Need no args\n");
		exit(1);
	}

	rval = read(STDIN_FILENO, &packet, sizeof(packet));
	if (rval < 0) {
		perror("local_ip: bad read");
		exit(1);
	}

	rawsock = socket(PF_INET, SOCK_RAW, packet.iph.protocol);
	if (rawsock < 0) {
		perror("local_ip: opening raw socket");
		exit(1);
	}

	if (setsockopt(rawsock, SOL_IP, IP_HDRINCL, &set, sizeof(set)) != 0) {
		perror("local_ip: setting IP_HDRINCL");
		exit(1);
	}

	if (write(rawsock, &packet, rval) != rval) {
		perror("local_ip: writing packet");
		exit(1);
	}
	close(rawsock);
	return 0;
}
