/* Simple netfilter thingy to pass packets. */
#include <sys/types.h>
#include <net/if.h>

#include "device/netfilter_dev.h"
#include <linux/netfilter_ipv4.h>

#include <fcntl.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>

#define NETFILTER_DEVICE_NAME 	"/dev/netfilter_ipv4"

int main(int argc, const char *argv[])
{
	int fd;
	struct nfdev_verdict verdict;
	struct nfdev_head head;

	if (argc > 2) {
		fprintf(stderr, "Usage: %s [hook name]\n", argv[0]);
		return 1;
	}

	fd = open(NETFILTER_DEVICE_NAME, O_RDWR);
	if (fd < 0) {
		fprintf(stderr, "opening %s: %s\n",
			NETFILTER_DEVICE_NAME, strerror(errno));
		return 1;
	}

	if (argv[1]) {
		struct nfdev_condition cond = {0xFFFFFFFF, 0, 0};

		/* Starts with "catch-all" condition. */
		if (ioctl(fd, NFDIOSDELCOND, &cond) < 0) {
			perror("deleting original condition");
			return 1;
		}

		/* FIXME: blow off any captured packets. --RR */
		if (strcasecmp(argv[1], "LOCAL_IN") == 0)
			cond.hookmask = 1 << NF_IP_LOCAL_IN;
		else if (strcasecmp(argv[1], "PRE_ROUTING") == 0)
			cond.hookmask = 1 << NF_IP_PRE_ROUTING;
		else if (strcasecmp(argv[1], "FORWARD") == 0)
			cond.hookmask = 1 << NF_IP_FORWARD;
		else if (strcasecmp(argv[1], "POST_ROUTING") == 0)
			cond.hookmask = 1 << NF_IP_POST_ROUTING;
		else if (strcasecmp(argv[1], "LOCAL_OUT") == 0)
			cond.hookmask = 1 << NF_IP_LOCAL_OUT;
		else {
			fprintf(stderr, "Unknown hook `%s'\n", argv[1]);
			return 1;
		}

		if (ioctl(fd, NFDIOSADDCOND, &cond) < 0) {
			perror("adding new condition");
			return 1;
		}
	}

	while (read(fd, &head, sizeof(head)) == sizeof(head)) {
#if 1
		printf("in=%s, out=%s, mk=%lu, len=%u\n",
		       head.iniface, head.outiface, head.mark, head.pktlen);
#endif
		verdict = ((struct nfdev_verdict)
		   { NF_ACCEPT, head.pktlen, head.mark, head.reason });
		if (ioctl(fd, NFDIOSVERDICT, &verdict) != 0) {
			fprintf(stderr, "ioctl: %s\n", strerror(errno));
			break;
		}
	}
	return 1;
}

	

