#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>

typedef struct pp_pattern *perproto_pattern_handle;

struct pp_pattern
{
	const unsigned char *pattern;
	size_t patternlen;

	size_t buffused;
	unsigned char buffer[0];
};

/* Register interest for a given data pattern on a given handle.
   Returns NULL for trouble. */
perproto_pattern_handle
perproto_pattern_register(const void *pattern, size_t size)
{
	struct pp_pattern *pathandle
		= malloc(sizeof(*pathandle) + size);

	if (pathandle)
		*pathandle = ((struct pp_pattern){ pattern, size, 0 });
	return pathandle;
}

/* Returns -1 or offset in packet. */
/* This implementation is naive.  Is it worth being smart? */
int
perproto_pattern_check(perproto_pattern_handle h,
		       const unsigned char *data, u_int32_t len)
{
	unsigned int i;

	/* Seek match in buffer + data */
	for (i = 0; i < h->buffused; i++) {
		if (memcmp(h->buffer + i, h->pattern, h->buffused - i)
		    == 0) {
			if (len < h->patternlen - (h->buffused-i)) {
				/* Packet shorter than buffer. */
				if (memcmp(data, h->pattern + (h->buffused-i),
					   len) == 0) {
					/* Append to buffer. */
					memmove(h->buffer, h->buffer + i,
						h->buffused - i);
					h->buffused -= i;
					memcpy(h->buffer + h->buffused, data,
					       len);
					h->buffused += len;
					return -1;
				}
			} else {
				/* Packet longer than buffer. */
				if (memcmp(data, h->pattern + (h->buffused-i),
					   h->patternlen - (h->buffused-i))
				    == 0) {
					i = h->patternlen - (h->buffused-i);
					h->buffused = 0;
					return i;
				}
			}
		}
	}

	/* No partial. */
	if (len > h->patternlen) {
		/* Do we match in this packet? */
		for (i = 0; i < len - h->patternlen; i++) {
			if (memcmp(data+i, h->pattern, h->patternlen) == 0) {
				h->buffused = 0;
				return i + h->patternlen;
			}
		}
	}

	/* Look for end match. */
	if (len < h->patternlen)
		i = 0;
	else
		i = len - h->patternlen;
	for (; i < len; i++) {
		if (memcmp(data + i, h->pattern, len - i) == 0) {
			memcpy(h->buffer, data+i, len - i);
			h->buffused = len - i;
			return -1;
		}
	}
	h->buffused = 0;
	return -1;
}

unsigned int divide(perproto_pattern_handle handle,
		    char *base, char buffer[], const char *pkt,
		    unsigned int pos, unsigned int count, unsigned int dot)
{
	unsigned int i;

	if (strlen(pkt) == 1) {
		unsigned int sofar = 0;
		int didmatch = 0;

		/* Finished dividing.  Hit it. */
		while (base != buffer) {
			int match = perproto_pattern_check(handle, base,
							   strlen(base));
			if (match >= 0) {
				if (sofar + match != pos)
					printf("Bad match: pos %u not %u!\n",
					       sofar + match, pos);
				didmatch = 1;
			}
			sofar += strlen(base);
			base += strlen(base)+1;
		}
		/* Last part of packet. */
		if (perproto_pattern_check(handle, pkt, 1) != -1)
			printf("Last part matched!\n");
		else if (!didmatch)
			printf("No match!\n");
		count++;
		if (count % dot == 0) printf(".");

	} else {
		for (i = 1; i < strlen(pkt); i++) {
			memcpy(buffer, pkt, i);
			buffer[i] = '\0';
			count = divide(handle, base, buffer + i + 1, pkt + i,
				       pos, count, dot);
		}
	}
	return count;
}

int main()
{
	char buffer[100];
	const char pattern[] = "PPORT ";
	const char pkt[] = "P\r\nPPO\r\nPPPORT xyz";
	unsigned int pos = strstr(pkt, pattern) - pkt + strlen(pattern);
	perproto_pattern_handle handle
		= perproto_pattern_register(pattern, strlen(pattern));

	/* Divide into parts in each combination.  Each one must match
           in the right point. */
	printf("\nNumber of iterations: %u\n",
	       divide(handle, buffer, buffer, pkt, pos, 0,
		      (1 << (strlen(pkt) - 1)) / 10));

	return 0;
}

/* How many ways can you divide up a string of length n?
   f(1) = P = 1.
   f(2) = P/O PO = 2.
   f(3) = P/O/R P/OR PO/R POR = 4
   f(4) = 1 + 2*f(3).
*/
